import _extends from '@babel/runtime/helpers/esm/extends';
import _asyncToGenerator from '@babel/runtime/helpers/esm/asyncToGenerator';
import _regeneratorRuntime from '@babel/runtime/regenerator';
import * as React from 'react';
import React__default from 'react';
import { EditorState } from '@codemirror/state';
import { EditorView } from '@codemirror/view';

// credit: https://github.com/JakeGinnivan/merge-refs-hook

function setRefs(ref, value) {
  if (typeof ref === 'function') {
    ref(value);
  } else if (ref) {
    ref.current = value;
  }
}

function useMergeRefs(forwardedRef, localRef) {
  return React__default.useCallback(value => {
    setRefs(forwardedRef, value);
    setRefs(localRef, value);
  }, [forwardedRef, localRef]);
}

var CodeMirror = /*#__PURE__*/React.forwardRef((_ref, ref) => {
  var value = _ref.value,
      onUpdate = _ref.onUpdate,
      _ref$extensions = _ref.extensions,
      passedExtensions = _ref$extensions === void 0 ? [] : _ref$extensions,
      elementProps = _ref.elementProps;
  var innerRef = React.useRef(null);
  var mergedRef = useMergeRefs(ref, innerRef);
  React.useEffect(() => {
    var currentEditor = innerRef.current;
    if (!currentEditor) return;
    var view;

    _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee() {
      var extensions, state;
      return _regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              extensions = [];
              if (onUpdate) extensions.push(EditorView.updateListener.of(onUpdate));
              state = EditorState.create({
                doc: value,
                extensions: [...extensions, ...passedExtensions]
              });
              view = new EditorView({
                state,
                parent: currentEditor
              });

            case 4:
            case "end":
              return _context.stop();
          }
        }
      }, _callee);
    }))();

    return () => view.destroy();
  }, [innerRef, passedExtensions]);
  return /*#__PURE__*/React.createElement("div", _extends({
    ref: mergedRef
  }, elementProps));
});
CodeMirror.displayName = 'CodeMirror';

export default CodeMirror;
